@@ -28,5 +28,5 @@ fn concurrent_isolate_creation_and_disposal() {
   }
 
   unsafe { v8::V8::dispose() };
-  v8::V8::shutdown_platform();
+  v8::V8::dispose_platform();
 }
