@@ -51,7 +51,11 @@ class V8_EXPORT EmbedderRootsHandler {
    * being treated as roots.
    */
   virtual bool IsRoot(const v8::TracedReference<v8::Value>& handle) = 0;
-  virtual bool IsRoot(const v8::TracedGlobal<v8::Value>& handle) = 0;
+
+  V8_DEPRECATE_SOON("See v8::TracedGlobal class comment.")
+  virtual bool IsRoot(const v8::TracedGlobal<v8::Value>& handle) {
+    return true;
+  }
 
   /**
    * Used in combination with |IsRoot|. Called by V8 when an
@@ -88,6 +92,7 @@ class V8_EXPORT EmbedderHeapTracer {
   class V8_EXPORT TracedGlobalHandleVisitor {
    public:
     virtual ~TracedGlobalHandleVisitor() = default;
+    V8_DEPRECATE_SOON("See v8::TracedGlobal class comment.")
     virtual void VisitTracedGlobalHandle(const TracedGlobal<Value>& handle) {}
     virtual void VisitTracedReference(const TracedReference<Value>& handle) {}
   };
@@ -124,14 +129,6 @@ class V8_EXPORT EmbedderHeapTracer {
    */
   void SetStackStart(void* stack_start);
 
-  /**
-   * Called by the embedder to notify V8 of an empty execution stack.
-   */
-  V8_DEPRECATED(
-      "This call only optimized internal caches which V8 is able to figure out "
-      "on its own now.")
-  void NotifyEmptyEmbedderStack();
-
   /**
    * Called by v8 to register internal fields of found wrappers.
    *
@@ -197,6 +194,7 @@ class V8_EXPORT EmbedderHeapTracer {
    */
   virtual bool IsRootForNonTracingGC(
       const v8::TracedReference<v8::Value>& handle);
+  V8_DEPRECATE_SOON("See v8::TracedGlobal class comment.")
   virtual bool IsRootForNonTracingGC(const v8::TracedGlobal<v8::Value>& handle);
 
   /**
@@ -210,7 +208,7 @@ class V8_EXPORT EmbedderHeapTracer {
    *
    * Should only be used in testing code.
    */
-  V8_DEPRECATE_SOON("Use Isolate::RequestGarbageCollectionForTesting instead")
+  V8_DEPRECATED("Use Isolate::RequestGarbageCollectionForTesting instead")
   void GarbageCollectionForTesting(EmbedderStackState stack_state);
 
   /*
