@@ -124,6 +124,8 @@ class V8_EXPORT_PRIVATE TurboAssembler : public TurboAssemblerBase {
   // Load the builtin given by the Smi in |builtin_index| into the same
   // register.
   void LoadEntryFromBuiltinIndex(Register builtin_index);
+  void LoadEntryFromBuiltin(Builtin builtin, Register destination);
+  MemOperand EntryFromBuiltinAsOperand(Builtin builtin);
   void LoadCodeObjectEntry(Register destination, Register code_object);
   void CallCodeObject(Register code_object);
   void JumpCodeObject(Register code_object,
@@ -136,6 +138,7 @@ class V8_EXPORT_PRIVATE TurboAssembler : public TurboAssemblerBase {
   void Move(Register dst, Handle<HeapObject> source,
             RelocInfo::Mode rmode = RelocInfo::FULL_EMBEDDED_OBJECT);
   void Move(Register dst, ExternalReference reference);
+  void Move(Register dst, const MemOperand& src);
   void Move(Register dst, Register src, Condition cond = al);
   void Move(DoubleRegister dst, DoubleRegister src);
 
@@ -238,6 +241,7 @@ class V8_EXPORT_PRIVATE TurboAssembler : public TurboAssemblerBase {
   // Add Logical (Register - Immediate)
   void AddU32(Register dst, const Operand& imm);
   void AddU64(Register dst, const Operand& imm);
+  void AddU64(Register dst, Register src1, Register src2);
 
   // Add Logical (Register - Mem)
   void AddU32(Register dst, const MemOperand& opnd);
@@ -1105,30 +1109,99 @@ class V8_EXPORT_PRIVATE TurboAssembler : public TurboAssemblerBase {
                         Register src2, uint8_t imm_lane_idx);
   void I8x16ReplaceLane(Simd128Register dst, Simd128Register src1,
                         Register src2, uint8_t imm_lane_idx);
+  void I64x2BitMask(Register dst, Simd128Register src, Register scratch1,
+                    Simd128Register scratch2);
+  void I32x4BitMask(Register dst, Simd128Register src, Register scratch1,
+                    Simd128Register scratch2);
+  void I16x8BitMask(Register dst, Simd128Register src, Register scratch1,
+                    Simd128Register scratch2);
+  void I8x16BitMask(Register dst, Simd128Register src, Register scratch1,
+                    Register scratch2, Simd128Register scratch3);
+  void V128AnyTrue(Register dst, Simd128Register src, Register scratch);
+  void I32x4SConvertF32x4(Simd128Register dst, Simd128Register src,
+                          Simd128Register scratch1, Register scratch2);
+  void I32x4UConvertF32x4(Simd128Register dst, Simd128Register src,
+                          Simd128Register scratch1, Register scratch2);
+  void F32x4SConvertI32x4(Simd128Register dst, Simd128Register src,
+                          Simd128Register scratch1, Register scratch2);
+  void F32x4UConvertI32x4(Simd128Register dst, Simd128Register src,
+                          Simd128Register scratch1, Register scratch2);
+  void I16x8SConvertI32x4(Simd128Register dst, Simd128Register src1,
+                          Simd128Register src2);
+  void I8x16SConvertI16x8(Simd128Register dst, Simd128Register src1,
+                          Simd128Register src2);
+  void I16x8UConvertI32x4(Simd128Register dst, Simd128Register src1,
+                          Simd128Register src2, Simd128Register scratch);
+  void I8x16UConvertI16x8(Simd128Register dst, Simd128Register src1,
+                          Simd128Register src2, Simd128Register scratch);
+  void F64x2PromoteLowF32x4(Simd128Register dst, Simd128Register src,
+                            Simd128Register scratch1, Register scratch2,
+                            Register scratch3, Register scratch4);
+  void F32x4DemoteF64x2Zero(Simd128Register dst, Simd128Register src,
+                            Simd128Register scratch1, Register scratch2,
+                            Register scratch3, Register scratch4);
+  void I32x4TruncSatF64x2SZero(Simd128Register dst, Simd128Register src,
+                               Simd128Register scratch);
+  void I32x4TruncSatF64x2UZero(Simd128Register dst, Simd128Register src,
+                               Simd128Register scratch);
+  void I8x16Swizzle(Simd128Register dst, Simd128Register src1,
+                    Simd128Register src2, Simd128Register scratch1,
+                    Simd128Register scratch2);
+  void S128Const(Simd128Register dst, uint64_t high, uint64_t low,
+                 Register scratch1, Register scratch2);
+  void I8x16Shuffle(Simd128Register dst, Simd128Register src1,
+                    Simd128Register src2, uint64_t high, uint64_t low,
+                    Register scratch1, Register scratch2,
+                    Simd128Register scratch3);
+  void I32x4DotI16x8S(Simd128Register dst, Simd128Register src1,
+                      Simd128Register src2, Simd128Register scratch);
+  void I16x8Q15MulRSatS(Simd128Register dst, Simd128Register src1,
+                        Simd128Register src2, Simd128Register scratch1,
+                        Simd128Register scratch2, Simd128Register scratch3);
+  void S128Select(Simd128Register dst, Simd128Register src1,
+                  Simd128Register src2, Simd128Register mask);
 
-#define SIMD_UNOP_LIST(V) \
-  V(F64x2Abs)             \
-  V(F64x2Neg)             \
-  V(F64x2Sqrt)            \
-  V(F64x2Ceil)            \
-  V(F64x2Floor)           \
-  V(F64x2Trunc)           \
-  V(F64x2NearestInt)      \
-  V(F32x4Abs)             \
-  V(F32x4Neg)             \
-  V(F32x4Sqrt)            \
-  V(F32x4Ceil)            \
-  V(F32x4Floor)           \
-  V(F32x4Trunc)           \
-  V(F32x4NearestInt)      \
-  V(I64x2Abs)             \
-  V(I32x4Abs)             \
-  V(I16x8Abs)             \
-  V(I8x16Abs)             \
-  V(I64x2Neg)             \
-  V(I32x4Neg)             \
-  V(I16x8Neg)             \
-  V(I8x16Neg)
+#define SIMD_UNOP_LIST(V)   \
+  V(F64x2Abs)               \
+  V(F64x2Neg)               \
+  V(F64x2Sqrt)              \
+  V(F64x2Ceil)              \
+  V(F64x2Floor)             \
+  V(F64x2Trunc)             \
+  V(F64x2NearestInt)        \
+  V(F64x2ConvertLowI32x4S)  \
+  V(F64x2ConvertLowI32x4U)  \
+  V(F32x4Abs)               \
+  V(F32x4Neg)               \
+  V(F32x4Sqrt)              \
+  V(F32x4Ceil)              \
+  V(F32x4Floor)             \
+  V(F32x4Trunc)             \
+  V(F32x4NearestInt)        \
+  V(I64x2Abs)               \
+  V(I64x2SConvertI32x4Low)  \
+  V(I64x2SConvertI32x4High) \
+  V(I64x2UConvertI32x4Low)  \
+  V(I64x2UConvertI32x4High) \
+  V(I64x2Neg)               \
+  V(I32x4Abs)               \
+  V(I32x4Neg)               \
+  V(I32x4SConvertI16x8Low)  \
+  V(I32x4SConvertI16x8High) \
+  V(I32x4UConvertI16x8Low)  \
+  V(I32x4UConvertI16x8High) \
+  V(I16x8Abs)               \
+  V(I16x8Neg)               \
+  V(I16x8SConvertI8x16Low)  \
+  V(I16x8SConvertI8x16High) \
+  V(I16x8UConvertI8x16Low)  \
+  V(I16x8UConvertI8x16High) \
+  V(I8x16Abs)               \
+  V(I8x16Neg)               \
+  V(I8x16Popcnt)            \
+  V(S128Not)                \
+  V(S128Zero)               \
+  V(S128AllOnes)
 
 #define PROTOTYPE_SIMD_UNOP(name) \
   void name(Simd128Register dst, Simd128Register src);
@@ -1136,96 +1209,106 @@ class V8_EXPORT_PRIVATE TurboAssembler : public TurboAssemblerBase {
 #undef PROTOTYPE_SIMD_UNOP
 #undef SIMD_UNOP_LIST
 
-#define SIMD_BINOP_LIST(V)      \
-  V(F64x2Add, Simd128Register)  \
-  V(F64x2Sub, Simd128Register)  \
-  V(F64x2Mul, Simd128Register)  \
-  V(F64x2Div, Simd128Register)  \
-  V(F64x2Min, Simd128Register)  \
-  V(F64x2Max, Simd128Register)  \
-  V(F64x2Eq, Simd128Register)   \
-  V(F64x2Ne, Simd128Register)   \
-  V(F64x2Lt, Simd128Register)   \
-  V(F64x2Le, Simd128Register)   \
-  V(F32x4Add, Simd128Register)  \
-  V(F32x4Sub, Simd128Register)  \
-  V(F32x4Mul, Simd128Register)  \
-  V(F32x4Div, Simd128Register)  \
-  V(F32x4Min, Simd128Register)  \
-  V(F32x4Max, Simd128Register)  \
-  V(F32x4Eq, Simd128Register)   \
-  V(F32x4Ne, Simd128Register)   \
-  V(F32x4Lt, Simd128Register)   \
-  V(F32x4Le, Simd128Register)   \
-  V(I64x2Add, Simd128Register)  \
-  V(I64x2Sub, Simd128Register)  \
-  V(I64x2Mul, Simd128Register)  \
-  V(I64x2Eq, Simd128Register)   \
-  V(I64x2Ne, Simd128Register)   \
-  V(I64x2GtS, Simd128Register)  \
-  V(I64x2GeS, Simd128Register)  \
-  V(I64x2Shl, Register)         \
-  V(I64x2ShrS, Register)        \
-  V(I64x2ShrU, Register)        \
-  V(I64x2Shl, const Operand&)   \
-  V(I64x2ShrS, const Operand&)  \
-  V(I64x2ShrU, const Operand&)  \
-  V(I32x4Add, Simd128Register)  \
-  V(I32x4Sub, Simd128Register)  \
-  V(I32x4Mul, Simd128Register)  \
-  V(I32x4Eq, Simd128Register)   \
-  V(I32x4Ne, Simd128Register)   \
-  V(I32x4GtS, Simd128Register)  \
-  V(I32x4GeS, Simd128Register)  \
-  V(I32x4GtU, Simd128Register)  \
-  V(I32x4GeU, Simd128Register)  \
-  V(I32x4MinS, Simd128Register) \
-  V(I32x4MinU, Simd128Register) \
-  V(I32x4MaxS, Simd128Register) \
-  V(I32x4MaxU, Simd128Register) \
-  V(I32x4Shl, Register)         \
-  V(I32x4ShrS, Register)        \
-  V(I32x4ShrU, Register)        \
-  V(I32x4Shl, const Operand&)   \
-  V(I32x4ShrS, const Operand&)  \
-  V(I32x4ShrU, const Operand&)  \
-  V(I16x8Add, Simd128Register)  \
-  V(I16x8Sub, Simd128Register)  \
-  V(I16x8Mul, Simd128Register)  \
-  V(I16x8Eq, Simd128Register)   \
-  V(I16x8Ne, Simd128Register)   \
-  V(I16x8GtS, Simd128Register)  \
-  V(I16x8GeS, Simd128Register)  \
-  V(I16x8GtU, Simd128Register)  \
-  V(I16x8GeU, Simd128Register)  \
-  V(I16x8MinS, Simd128Register) \
-  V(I16x8MinU, Simd128Register) \
-  V(I16x8MaxS, Simd128Register) \
-  V(I16x8MaxU, Simd128Register) \
-  V(I16x8Shl, Register)         \
-  V(I16x8ShrS, Register)        \
-  V(I16x8ShrU, Register)        \
-  V(I16x8Shl, const Operand&)   \
-  V(I16x8ShrS, const Operand&)  \
-  V(I16x8ShrU, const Operand&)  \
-  V(I8x16Add, Simd128Register)  \
-  V(I8x16Sub, Simd128Register)  \
-  V(I8x16Eq, Simd128Register)   \
-  V(I8x16Ne, Simd128Register)   \
-  V(I8x16GtS, Simd128Register)  \
-  V(I8x16GeS, Simd128Register)  \
-  V(I8x16GtU, Simd128Register)  \
-  V(I8x16GeU, Simd128Register)  \
-  V(I8x16MinS, Simd128Register) \
-  V(I8x16MinU, Simd128Register) \
-  V(I8x16MaxS, Simd128Register) \
-  V(I8x16MaxU, Simd128Register) \
-  V(I8x16Shl, Register)         \
-  V(I8x16ShrS, Register)        \
-  V(I8x16ShrU, Register)        \
-  V(I8x16Shl, const Operand&)   \
-  V(I8x16ShrS, const Operand&)  \
-  V(I8x16ShrU, const Operand&)
+#define SIMD_BINOP_LIST(V)                  \
+  V(F64x2Add, Simd128Register)              \
+  V(F64x2Sub, Simd128Register)              \
+  V(F64x2Mul, Simd128Register)              \
+  V(F64x2Div, Simd128Register)              \
+  V(F64x2Min, Simd128Register)              \
+  V(F64x2Max, Simd128Register)              \
+  V(F64x2Eq, Simd128Register)               \
+  V(F64x2Ne, Simd128Register)               \
+  V(F64x2Lt, Simd128Register)               \
+  V(F64x2Le, Simd128Register)               \
+  V(F64x2Pmin, Simd128Register)             \
+  V(F64x2Pmax, Simd128Register)             \
+  V(F32x4Add, Simd128Register)              \
+  V(F32x4Sub, Simd128Register)              \
+  V(F32x4Mul, Simd128Register)              \
+  V(F32x4Div, Simd128Register)              \
+  V(F32x4Min, Simd128Register)              \
+  V(F32x4Max, Simd128Register)              \
+  V(F32x4Eq, Simd128Register)               \
+  V(F32x4Ne, Simd128Register)               \
+  V(F32x4Lt, Simd128Register)               \
+  V(F32x4Le, Simd128Register)               \
+  V(F32x4Pmin, Simd128Register)             \
+  V(F32x4Pmax, Simd128Register)             \
+  V(I64x2Add, Simd128Register)              \
+  V(I64x2Sub, Simd128Register)              \
+  V(I64x2Mul, Simd128Register)              \
+  V(I64x2Eq, Simd128Register)               \
+  V(I64x2Ne, Simd128Register)               \
+  V(I64x2GtS, Simd128Register)              \
+  V(I64x2GeS, Simd128Register)              \
+  V(I64x2Shl, Register)                     \
+  V(I64x2ShrS, Register)                    \
+  V(I64x2ShrU, Register)                    \
+  V(I64x2Shl, const Operand&)               \
+  V(I64x2ShrS, const Operand&)              \
+  V(I64x2ShrU, const Operand&)              \
+  V(I32x4Add, Simd128Register)              \
+  V(I32x4Sub, Simd128Register)              \
+  V(I32x4Mul, Simd128Register)              \
+  V(I32x4Eq, Simd128Register)               \
+  V(I32x4Ne, Simd128Register)               \
+  V(I32x4GtS, Simd128Register)              \
+  V(I32x4GeS, Simd128Register)              \
+  V(I32x4GtU, Simd128Register)              \
+  V(I32x4GeU, Simd128Register)              \
+  V(I32x4MinS, Simd128Register)             \
+  V(I32x4MinU, Simd128Register)             \
+  V(I32x4MaxS, Simd128Register)             \
+  V(I32x4MaxU, Simd128Register)             \
+  V(I32x4Shl, Register)                     \
+  V(I32x4ShrS, Register)                    \
+  V(I32x4ShrU, Register)                    \
+  V(I32x4Shl, const Operand&)               \
+  V(I32x4ShrS, const Operand&)              \
+  V(I32x4ShrU, const Operand&)              \
+  V(I16x8Add, Simd128Register)              \
+  V(I16x8Sub, Simd128Register)              \
+  V(I16x8Mul, Simd128Register)              \
+  V(I16x8Eq, Simd128Register)               \
+  V(I16x8Ne, Simd128Register)               \
+  V(I16x8GtS, Simd128Register)              \
+  V(I16x8GeS, Simd128Register)              \
+  V(I16x8GtU, Simd128Register)              \
+  V(I16x8GeU, Simd128Register)              \
+  V(I16x8MinS, Simd128Register)             \
+  V(I16x8MinU, Simd128Register)             \
+  V(I16x8MaxS, Simd128Register)             \
+  V(I16x8MaxU, Simd128Register)             \
+  V(I16x8Shl, Register)                     \
+  V(I16x8ShrS, Register)                    \
+  V(I16x8ShrU, Register)                    \
+  V(I16x8Shl, const Operand&)               \
+  V(I16x8ShrS, const Operand&)              \
+  V(I16x8ShrU, const Operand&)              \
+  V(I16x8RoundingAverageU, Simd128Register) \
+  V(I8x16Add, Simd128Register)              \
+  V(I8x16Sub, Simd128Register)              \
+  V(I8x16Eq, Simd128Register)               \
+  V(I8x16Ne, Simd128Register)               \
+  V(I8x16GtS, Simd128Register)              \
+  V(I8x16GeS, Simd128Register)              \
+  V(I8x16GtU, Simd128Register)              \
+  V(I8x16GeU, Simd128Register)              \
+  V(I8x16MinS, Simd128Register)             \
+  V(I8x16MinU, Simd128Register)             \
+  V(I8x16MaxS, Simd128Register)             \
+  V(I8x16MaxU, Simd128Register)             \
+  V(I8x16Shl, Register)                     \
+  V(I8x16ShrS, Register)                    \
+  V(I8x16ShrU, Register)                    \
+  V(I8x16Shl, const Operand&)               \
+  V(I8x16ShrS, const Operand&)              \
+  V(I8x16ShrU, const Operand&)              \
+  V(I8x16RoundingAverageU, Simd128Register) \
+  V(S128And, Simd128Register)               \
+  V(S128Or, Simd128Register)                \
+  V(S128Xor, Simd128Register)               \
+  V(S128AndNot, Simd128Register)
 
 #define PROTOTYPE_SIMD_BINOP(name, stype) \
   void name(Simd128Register dst, Simd128Register src1, stype src2);
@@ -1233,6 +1316,83 @@ class V8_EXPORT_PRIVATE TurboAssembler : public TurboAssemblerBase {
 #undef PROTOTYPE_SIMD_BINOP
 #undef SIMD_BINOP_LIST
 
+#define SIMD_EXT_MUL_LIST(V) \
+  V(I64x2ExtMulLowI32x4S)    \
+  V(I64x2ExtMulHighI32x4S)   \
+  V(I64x2ExtMulLowI32x4U)    \
+  V(I64x2ExtMulHighI32x4U)   \
+  V(I32x4ExtMulLowI16x8S)    \
+  V(I32x4ExtMulHighI16x8S)   \
+  V(I32x4ExtMulLowI16x8U)    \
+  V(I32x4ExtMulHighI16x8U)   \
+  V(I16x8ExtMulLowI8x16S)    \
+  V(I16x8ExtMulHighI8x16S)   \
+  V(I16x8ExtMulLowI8x16U)    \
+  V(I16x8ExtMulHighI8x16U)
+
+#define PROTOTYPE_SIMD_EXT_MUL(name)                                         \
+  void name(Simd128Register dst, Simd128Register src1, Simd128Register src2, \
+            Simd128Register scratch);
+  SIMD_EXT_MUL_LIST(PROTOTYPE_SIMD_EXT_MUL)
+#undef PROTOTYPE_SIMD_EXT_MUL
+#undef SIMD_EXT_MUL_LIST
+
+#define SIMD_ALL_TRUE_LIST(V) \
+  V(I64x2AllTrue)             \
+  V(I32x4AllTrue)             \
+  V(I16x8AllTrue)             \
+  V(I8x16AllTrue)
+
+#define PROTOTYPE_SIMD_ALL_TRUE(name)                             \
+  void name(Register dst, Simd128Register src, Register scratch1, \
+            Simd128Register scratch2);
+  SIMD_ALL_TRUE_LIST(PROTOTYPE_SIMD_ALL_TRUE)
+#undef PROTOTYPE_SIMD_ALL_TRUE
+#undef SIMD_ALL_TRUE_LIST
+
+#define SIMD_QFM_LIST(V) \
+  V(F64x2Qfma)           \
+  V(F64x2Qfms)           \
+  V(F32x4Qfma)           \
+  V(F32x4Qfms)
+
+#define PROTOTYPE_SIMD_QFM(name)                                             \
+  void name(Simd128Register dst, Simd128Register src1, Simd128Register src2, \
+            Simd128Register src3);
+  SIMD_QFM_LIST(PROTOTYPE_SIMD_QFM)
+#undef PROTOTYPE_SIMD_QFM
+#undef SIMD_QFM_LIST
+
+#define SIMD_ADD_SUB_SAT_LIST(V) \
+  V(I16x8AddSatS)                \
+  V(I16x8SubSatS)                \
+  V(I16x8AddSatU)                \
+  V(I16x8SubSatU)                \
+  V(I8x16AddSatS)                \
+  V(I8x16SubSatS)                \
+  V(I8x16AddSatU)                \
+  V(I8x16SubSatU)
+
+#define PROTOTYPE_SIMD_ADD_SUB_SAT(name)                                     \
+  void name(Simd128Register dst, Simd128Register src1, Simd128Register src2, \
+            Simd128Register scratch1, Simd128Register scratch2);
+  SIMD_ADD_SUB_SAT_LIST(PROTOTYPE_SIMD_ADD_SUB_SAT)
+#undef PROTOTYPE_SIMD_ADD_SUB_SAT
+#undef SIMD_ADD_SUB_SAT_LIST
+
+#define SIMD_EXT_ADD_PAIRWISE_LIST(V) \
+  V(I32x4ExtAddPairwiseI16x8S)        \
+  V(I32x4ExtAddPairwiseI16x8U)        \
+  V(I16x8ExtAddPairwiseI8x16S)        \
+  V(I16x8ExtAddPairwiseI8x16U)
+
+#define PROTOTYPE_SIMD_EXT_ADD_PAIRWISE(name)         \
+  void name(Simd128Register dst, Simd128Register src, \
+            Simd128Register scratch1, Simd128Register scratch2);
+  SIMD_EXT_ADD_PAIRWISE_LIST(PROTOTYPE_SIMD_EXT_ADD_PAIRWISE)
+#undef PROTOTYPE_SIMD_EXT_ADD_PAIRWISE
+#undef SIMD_EXT_ADD_PAIRWISE_LIST
+
   // ---------------------------------------------------------------------------
   // Pointer compression Support
 
@@ -1251,6 +1411,7 @@ class V8_EXPORT_PRIVATE TurboAssembler : public TurboAssemblerBase {
   void LoadTaggedPointerField(const Register& destination,
                               const MemOperand& field_operand,
                               const Register& scratch = no_reg);
+  void LoadTaggedSignedField(Register destination, MemOperand field_operand);
 
   // Loads a field containing any tagged value and decompresses it if necessary.
   void LoadAnyTaggedField(const Register& destination,
