@@ -89,13 +89,16 @@ void CppHeap::EnableDetachedGarbageCollectionsForTesting() {
 }
 
 void CppHeap::CollectGarbageForTesting(cppgc::EmbedderStackState stack_state) {
-  return internal::CppHeap::From(this)->CollectGarbageForTesting(stack_state);
+  return internal::CppHeap::From(this)->CollectGarbageForTesting(
+      cppgc::internal::GarbageCollector::Config::CollectionType::kMajor,
+      stack_state);
 }
 
-void JSHeapConsistency::CheckWrapper(v8::Local<v8::Object>& wrapper,
-                                     int wrapper_index, const void* wrappable) {
-  CHECK_EQ(wrappable,
-           wrapper->GetAlignedPointerFromInternalField(wrapper_index));
+void CppHeap::CollectGarbageInYoungGenerationForTesting(
+    cppgc::EmbedderStackState stack_state) {
+  return internal::CppHeap::From(this)->CollectGarbageForTesting(
+      cppgc::internal::GarbageCollector::Config::CollectionType::kMinor,
+      stack_state);
 }
 
 namespace internal {
@@ -193,11 +196,11 @@ class UnifiedHeapMarker final : public cppgc::internal::MarkerBase {
   }
 
   cppgc::internal::MarkingStateBase& GetMutatorMarkingState() {
-    return marking_visitor_.marking_state_;
+    return marking_visitor_->marking_state_;
   }
 
  protected:
-  cppgc::Visitor& visitor() final { return marking_visitor_; }
+  cppgc::Visitor& visitor() final { return *marking_visitor_; }
   cppgc::internal::ConservativeTracingVisitor& conservative_visitor() final {
     return conservative_marking_visitor_;
   }
@@ -207,7 +210,7 @@ class UnifiedHeapMarker final : public cppgc::internal::MarkerBase {
 
  private:
   UnifiedHeapMarkingState unified_heap_marking_state_;
-  MutatorUnifiedHeapMarkingVisitor marking_visitor_;
+  std::unique_ptr<MutatorUnifiedHeapMarkingVisitor> marking_visitor_;
   cppgc::internal::ConservativeMarkingVisitor conservative_marking_visitor_;
 };
 
@@ -217,10 +220,15 @@ UnifiedHeapMarker::UnifiedHeapMarker(Heap* v8_heap,
                                      MarkingConfig config)
     : cppgc::internal::MarkerBase(heap, platform, config),
       unified_heap_marking_state_(v8_heap),
-      marking_visitor_(heap, mutator_marking_state_,
-                       unified_heap_marking_state_),
+      marking_visitor_(
+          config.collection_type == cppgc::internal::GarbageCollector::Config::
+                                        CollectionType::kMajor
+              ? std::make_unique<MutatorUnifiedHeapMarkingVisitor>(
+                    heap, mutator_marking_state_, unified_heap_marking_state_)
+              : std::make_unique<MutatorMinorGCMarkingVisitor>(
+                    heap, mutator_marking_state_, unified_heap_marking_state_)),
       conservative_marking_visitor_(heap, mutator_marking_state_,
-                                    marking_visitor_) {
+                                    *marking_visitor_) {
   concurrent_marker_ = std::make_unique<UnifiedHeapConcurrentMarker>(
       heap_, marking_worklists_, schedule_, platform_,
       unified_heap_marking_state_);
@@ -235,7 +243,6 @@ void CppHeap::MetricRecorderAdapter::AddMainThreadEvent(
     const FullCycle& cppgc_event) {
   DCHECK(!last_full_gc_event_.has_value());
   last_full_gc_event_ = cppgc_event;
-  GetIsolate()->heap()->tracer()->NotifyGCCompleted();
 }
 
 void CppHeap::MetricRecorderAdapter::AddMainThreadEvent(
@@ -428,7 +435,9 @@ bool ShouldReduceMemory(CppHeap::GarbageCollectionFlags flags) {
 
 }  // namespace
 
-void CppHeap::InitializeTracing(GarbageCollectionFlags gc_flags) {
+void CppHeap::InitializeTracing(
+    cppgc::internal::GarbageCollector::Config::CollectionType collection_type,
+    GarbageCollectionFlags gc_flags) {
   CHECK(!sweeper_.IsSweepingInProgress());
 
   // Check that previous cycle metrics have been reported.
@@ -436,14 +445,15 @@ void CppHeap::InitializeTracing(GarbageCollectionFlags gc_flags) {
                  !GetMetricRecorder()->MetricsReportPending());
 
 #if defined(CPPGC_YOUNG_GENERATION)
-  cppgc::internal::SequentialUnmarker unmarker(raw_heap());
+  if (collection_type ==
+      cppgc::internal::GarbageCollector::Config::CollectionType::kMajor)
+    cppgc::internal::SequentialUnmarker unmarker(raw_heap());
 #endif  // defined(CPPGC_YOUNG_GENERATION)
 
   current_gc_flags_ = gc_flags;
 
   const UnifiedHeapMarker::MarkingConfig marking_config{
-      UnifiedHeapMarker::MarkingConfig::CollectionType::kMajor,
-      cppgc::Heap::StackState::kNoHeapPointers,
+      collection_type, cppgc::Heap::StackState::kNoHeapPointers,
       (IsForceGC(current_gc_flags_) && !force_incremental_marking_for_testing_)
           ? UnifiedHeapMarker::MarkingConfig::MarkingType::kAtomic
           : UnifiedHeapMarker::MarkingConfig::MarkingType::
@@ -503,7 +513,8 @@ void CppHeap::EnterFinalPause(cppgc::EmbedderStackState stack_state) {
                                       stack_state);
 }
 
-void CppHeap::TraceEpilogue() {
+void CppHeap::TraceEpilogue(
+    cppgc::internal::GarbageCollector::Config::CollectionType collection_type) {
   CHECK(in_atomic_pause_);
   CHECK(marking_done_);
   {
@@ -523,8 +534,7 @@ void CppHeap::TraceEpilogue() {
   buffered_allocated_bytes_ = 0;
   const size_t bytes_allocated_in_prefinalizers = ExecutePreFinalizers();
 #if CPPGC_VERIFY_HEAP
-  UnifiedHeapMarkingVerifier verifier(
-      *this, cppgc::internal::Heap::Config::CollectionType::kMajor);
+  UnifiedHeapMarkingVerifier verifier(*this, collection_type);
   verifier.Run(
       stack_state_of_prev_gc(), stack_end_of_current_gc(),
       stats_collector()->marked_bytes() + bytes_allocated_in_prefinalizers);
@@ -596,6 +606,7 @@ void CppHeap::ReportBufferedAllocationSizeIfPossible() {
 }
 
 void CppHeap::CollectGarbageForTesting(
+    cppgc::internal::GarbageCollector::Config::CollectionType collection_type,
     cppgc::internal::GarbageCollector::Config::StackState stack_state) {
   if (in_no_gc_scope()) return;
 
@@ -612,12 +623,12 @@ void CppHeap::CollectGarbageForTesting(
     // Perform an atomic GC, with starting incremental/concurrent marking and
     // immediately finalizing the garbage collection.
     if (!IsMarking()) {
-      InitializeTracing(GarbageCollectionFlagValues::kForced);
+      InitializeTracing(collection_type, GarbageCollectionFlagValues::kForced);
       StartTracing();
     }
     EnterFinalPause(stack_state);
     AdvanceTracing(std::numeric_limits<double>::infinity());
-    TraceEpilogue();
+    TraceEpilogue(collection_type);
   }
 }
 
@@ -635,7 +646,9 @@ void CppHeap::StartIncrementalGarbageCollectionForTesting() {
   DCHECK_NULL(isolate_);
   if (IsMarking()) return;
   force_incremental_marking_for_testing_ = true;
-  InitializeTracing(GarbageCollectionFlagValues::kForced);
+  InitializeTracing(
+      cppgc::internal::GarbageCollector::Config::CollectionType::kMajor,
+      GarbageCollectionFlagValues::kForced);
   StartTracing();
   force_incremental_marking_for_testing_ = false;
 }
@@ -646,7 +659,9 @@ void CppHeap::FinalizeIncrementalGarbageCollectionForTesting(
   DCHECK_NULL(isolate_);
   DCHECK(IsMarking());
   if (IsMarking()) {
-    CollectGarbageForTesting(stack_state);
+    CollectGarbageForTesting(
+        cppgc::internal::GarbageCollector::Config::CollectionType::kMajor,
+        stack_state);
   }
   sweeper_.FinishIfRunning();
 }
