@@ -95,9 +95,9 @@ bool PagedSpace::EnsureLabMain(int size_in_bytes, AllocationOrigin origin) {
 
 AllocationResult PagedSpace::AllocateFastUnaligned(int size_in_bytes) {
   if (!allocation_info_->CanIncrementTop(size_in_bytes)) {
-    return AllocationResult::Retry(identity());
+    return AllocationResult::Failure(identity());
   }
-  return AllocationResult(
+  return AllocationResult::FromObject(
       HeapObject::FromAddress(allocation_info_->IncrementTop(size_in_bytes)));
 }
 
@@ -108,7 +108,7 @@ AllocationResult PagedSpace::AllocateFastAligned(
   int filler_size = Heap::GetFillToAlign(current_top, alignment);
   int aligned_size = filler_size + size_in_bytes;
   if (!allocation_info_->CanIncrementTop(aligned_size)) {
-    return AllocationResult::Retry(identity());
+    return AllocationResult::Failure(identity());
   }
   HeapObject obj =
       HeapObject::FromAddress(allocation_info_->IncrementTop(aligned_size));
@@ -116,18 +116,18 @@ AllocationResult PagedSpace::AllocateFastAligned(
   if (filler_size > 0) {
     obj = heap()->PrecedeWithFiller(obj, filler_size);
   }
-  return AllocationResult(obj);
+  return AllocationResult::FromObject(obj);
 }
 
 AllocationResult PagedSpace::AllocateRawUnaligned(int size_in_bytes,
                                                   AllocationOrigin origin) {
   DCHECK(!FLAG_enable_third_party_heap);
   if (!EnsureLabMain(size_in_bytes, origin)) {
-    return AllocationResult::Retry(identity());
+    return AllocationResult::Failure(identity());
   }
 
   AllocationResult result = AllocateFastUnaligned(size_in_bytes);
-  DCHECK(!result.IsRetry());
+  DCHECK(!result.IsFailure());
   MSAN_ALLOCATED_UNINITIALIZED_MEMORY(result.ToObjectChecked().address(),
                                       size_in_bytes);
 
@@ -152,12 +152,12 @@ AllocationResult PagedSpace::AllocateRawAligned(int size_in_bytes,
   int filler_size = Heap::GetMaximumFillToAlign(alignment);
   allocation_size += filler_size;
   if (!EnsureLabMain(allocation_size, origin)) {
-    return AllocationResult::Retry(identity());
+    return AllocationResult::Failure(identity());
   }
   int aligned_size_in_bytes;
   AllocationResult result =
       AllocateFastAligned(size_in_bytes, &aligned_size_in_bytes, alignment);
-  DCHECK(!result.IsRetry());
+  DCHECK(!result.IsFailure());
   MSAN_ALLOCATED_UNINITIALIZED_MEMORY(result.ToObjectChecked().address(),
                                       size_in_bytes);
 
@@ -183,11 +183,8 @@ AllocationResult PagedSpace::AllocateRaw(int size_in_bytes,
     result = AllocateFastUnaligned(size_in_bytes);
   }
 
-  if (!result.IsRetry()) {
-    return result;
-  } else {
-    return AllocateRawSlow(size_in_bytes, alignment, origin);
-  }
+  return result.IsFailure() ? AllocateRawSlow(size_in_bytes, alignment, origin)
+                            : result;
 }
 
 }  // namespace internal
