@@ -165,18 +165,14 @@ void Sweeper::StartSweeping() {
     // evacuating a page, already swept pages will have enough free bytes to
     // hold the objects to move (and therefore, we won't need to wait for more
     // pages to be swept in order to move those objects).
-    // Since maps don't move, there is no need to sort the pages from MAP_SPACE
-    // before sweeping them.
     // We sort in descending order of live bytes, i.e., ascending order of free
     // bytes, because GetSweepingPageSafe returns pages in reverse order.
-    if (space != MAP_SPACE) {
-      int space_index = GetSweepSpaceIndex(space);
-      std::sort(
-          sweeping_list_[space_index].begin(),
-          sweeping_list_[space_index].end(), [marking_state](Page* a, Page* b) {
-            return marking_state->live_bytes(a) > marking_state->live_bytes(b);
-          });
-    }
+    int space_index = GetSweepSpaceIndex(space);
+    std::sort(
+        sweeping_list_[space_index].begin(), sweeping_list_[space_index].end(),
+        [marking_state](Page* a, Page* b) {
+          return marking_state->live_bytes(a) > marking_state->live_bytes(b);
+        });
   });
 }
 
@@ -387,7 +383,8 @@ int Sweeper::RawSweep(
           &old_to_new_cleanup);
     }
     Map map = object.map(cage_base, kAcquireLoad);
-    DCHECK(map.IsMap(cage_base));
+    // Map might be forwarded during GC.
+    DCHECK(MarkCompactCollector::IsMapOrForwardedMap(map));
     int size = object.SizeFromMap(map);
     live_bytes += size;
     free_start = free_end + size;
