@@ -14,6 +14,7 @@
 #include "src/objects/call-site-info.h"
 #include "src/objects/cell.h"
 #include "src/objects/data-handler.h"
+#include "src/objects/embedder-data-array-inl.h"
 #include "src/objects/fixed-array.h"
 #include "src/objects/foreign-inl.h"
 #include "src/objects/free-space-inl.h"
@@ -106,6 +107,11 @@ void BodyDescriptorBase::IterateJSObjectBodyImpl(Map map, HeapObject obj,
   STATIC_ASSERT(kEmbedderDataSlotSize == kTaggedSize);
 #endif
   IteratePointers(obj, start_offset, end_offset, v);
+
+  JSObject js_obj = JSObject::cast(obj);
+  for (int i = 0; i < js_obj.GetEmbedderFieldCount(); i++) {
+    v->VisitEmbedderDataSlot(obj, EmbedderDataSlot(js_obj, i));
+  }
 }
 
 template <typename ObjectVisitor>
@@ -635,6 +641,8 @@ class Foreign::BodyDescriptor final : public BodyDescriptorBase {
     v->VisitExternalReference(
         Foreign::cast(obj), reinterpret_cast<Address*>(
                                 obj.RawField(kForeignAddressOffset).address()));
+    v->VisitExternalPointer(obj,
+                            obj.RawExternalPointerField(kForeignAddressOffset));
   }
 
   static inline int SizeOf(Map map, HeapObject object) { return kSize; }
@@ -783,7 +791,14 @@ class ExternalOneByteString::BodyDescriptor final : public BodyDescriptorBase {
 
   template <typename ObjectVisitor>
   static inline void IterateBody(Map map, HeapObject obj, int object_size,
-                                 ObjectVisitor* v) {}
+                                 ObjectVisitor* v) {
+    ExternalString string = ExternalString::cast(obj);
+    v->VisitExternalPointer(obj,
+                            string.RawExternalPointerField(kResourceOffset));
+    if (string.is_uncached()) return;
+    v->VisitExternalPointer(
+        obj, string.RawExternalPointerField(kResourceDataOffset));
+  }
 
   static inline int SizeOf(Map map, HeapObject object) { return kSize; }
 };
@@ -794,7 +809,14 @@ class ExternalTwoByteString::BodyDescriptor final : public BodyDescriptorBase {
 
   template <typename ObjectVisitor>
   static inline void IterateBody(Map map, HeapObject obj, int object_size,
-                                 ObjectVisitor* v) {}
+                                 ObjectVisitor* v) {
+    ExternalString string = ExternalString::cast(obj);
+    v->VisitExternalPointer(obj,
+                            string.RawExternalPointerField(kResourceOffset));
+    if (string.is_uncached()) return;
+    v->VisitExternalPointer(
+        obj, string.RawExternalPointerField(kResourceDataOffset));
+  }
 
   static inline int SizeOf(Map map, HeapObject object) { return kSize; }
 };
@@ -916,6 +938,8 @@ class NativeContext::BodyDescriptor final : public BodyDescriptorBase {
                     NativeContext::kEndOfStrongFieldsOffset, v);
     IterateCustomWeakPointers(obj, NativeContext::kStartOfWeakFieldsOffset,
                               NativeContext::kEndOfWeakFieldsOffset, v);
+    v->VisitExternalPointer(obj,
+                            obj.RawExternalPointerField(kMicrotaskQueueOffset));
   }
 
   static inline int SizeOf(Map map, HeapObject object) {
@@ -941,6 +965,8 @@ class CodeDataContainer::BodyDescriptor final : public BodyDescriptorBase {
 
     if (V8_EXTERNAL_CODE_SPACE_BOOL) {
       v->VisitCodePointer(obj, obj.RawCodeField(kCodeOffset));
+      v->VisitExternalPointer(
+          obj, obj.RawExternalPointerField(kCodeEntryPointOffset));
     }
   }
 
@@ -984,6 +1010,13 @@ class EmbedderDataArray::BodyDescriptor final : public BodyDescriptorBase {
     STATIC_ASSERT(kEmbedderDataSlotSize == kTaggedSize);
     IteratePointers(obj, EmbedderDataArray::kHeaderSize, object_size, v);
 #endif
+
+    EmbedderDataArray array = EmbedderDataArray::cast(obj);
+    EmbedderDataSlot start(array, 0);
+    EmbedderDataSlot end(array, array.length());
+    for (EmbedderDataSlot slot = start; slot < end; ++slot) {
+      v->VisitEmbedderDataSlot(obj, slot);
+    }
   }
 
   static inline int SizeOf(Map map, HeapObject object) {
@@ -1078,6 +1111,8 @@ auto BodyDescriptorApply(InstanceType type, Args&&... args) {
       return CALL_APPLY(WasmInternalFunction);
     case WASM_JS_FUNCTION_DATA_TYPE:
       return CALL_APPLY(WasmJSFunctionData);
+    case WASM_ON_FULFILLED_DATA_TYPE:
+      return CALL_APPLY(WasmOnFulfilledData);
     case WASM_STRUCT_TYPE:
       return CALL_APPLY(WasmStruct);
     case WASM_TYPE_INFO_TYPE:
@@ -1121,6 +1156,7 @@ auto BodyDescriptorApply(InstanceType type, Args&&... args) {
     case JS_SET_TYPE:
     case JS_SET_VALUE_ITERATOR_TYPE:
     case JS_SPECIAL_API_OBJECT_TYPE:
+    case JS_SHADOW_REALM_TYPE:
     case JS_STRING_ITERATOR_PROTOTYPE_TYPE:
     case JS_STRING_ITERATOR_TYPE:
     case JS_TEMPORAL_CALENDAR_TYPE:
@@ -1265,6 +1301,12 @@ void HeapObject::IterateFast(PtrComprCageBase cage_base, ObjectVisitor* v) {
   IterateBodyFast(cage_base, v);
 }
 
+template <typename ObjectVisitor>
+void HeapObject::IterateFast(Map map, int object_size, ObjectVisitor* v) {
+  v->VisitMapPointer(*this);
+  IterateBodyFast(map, object_size, v);
+}
+
 template <typename ObjectVisitor>
 void HeapObject::IterateBodyFast(PtrComprCageBase cage_base, ObjectVisitor* v) {
   Map m = map(cage_base);
