@@ -288,7 +288,7 @@ StringMigrationResult MigrateStringMapUnderLockIfNeeded(
     CHECK(string.release_compare_and_swap_map_word(
         MapWord::FromMap(sentinel_map), MapWord::FromMap(target_map)));
   } else {
-    string.set_map(target_map, kReleaseStore);
+    string.set_map_safe_transition(target_map, kReleaseStore);
   }
 
   return StringMigrationResult::kThisThreadMigrated;
@@ -541,6 +541,10 @@ bool String::SupportsExternalization() {
     return false;
   }
 
+  // External strings in the shared heap conflicts with the heap sandbox at the
+  // moment. Disable it until supported.
+  if (InSharedHeap()) return false;
+
 #ifdef V8_COMPRESS_POINTERS
   // Small strings may not be in-place externalizable.
   if (this->Size() < ExternalString::kUncachedSize) return false;
@@ -1632,7 +1636,7 @@ uint32_t String::ComputeAndSetHash(
 
   // Check the hash code is there.
   DCHECK(HasHashCode());
-  uint32_t result = raw_hash_field >> kHashShift;
+  uint32_t result = HashBits::decode(raw_hash_field);
   DCHECK_NE(result, 0);  // Ensure that the hash value of 0 is never computed.
   return result;
 }
@@ -1643,7 +1647,7 @@ bool String::SlowAsArrayIndex(uint32_t* index) {
   if (length <= kMaxCachedArrayIndexLength) {
     EnsureHash();  // Force computation of hash code.
     uint32_t field = raw_hash_field();
-    if ((field & kIsNotIntegerIndexMask) != 0) return false;
+    if (!IsIntegerIndex(field)) return false;
     *index = ArrayIndexValueBits::decode(field);
     return true;
   }
@@ -1658,7 +1662,7 @@ bool String::SlowAsIntegerIndex(size_t* index) {
   if (length <= kMaxCachedArrayIndexLength) {
     EnsureHash();  // Force computation of hash code.
     uint32_t field = raw_hash_field();
-    if ((field & kIsNotIntegerIndexMask) != 0) return false;
+    if (!IsIntegerIndex(field)) return false;
     *index = ArrayIndexValueBits::decode(field);
     return true;
   }
